use std::{fs::File, io};

use crate::utils::input_reader;

enum Engine {
    Symbol(char),
    Number(u32),
    Empty,
}

pub fn part_one() -> String {
    if let Ok(lines) = input_reader::read_lines("./input/2023/day_03/input.txt") {
        let matrix = create_matrix(lines);
        let result = process_matrix(&matrix);

        result.to_string()
    } else {
        panic!("Failed to read lines from input file");
    }
}


fn create_matrix(lines: io::Lines<io::BufReader<File>>) -> Vec<Vec<Engine>> {
    let mut matrix: Vec<Vec<Engine>> = Vec::new();

    for (i, line) in lines.enumerate() {
        matrix.push(Vec::new());

        let line = line.unwrap();

        for ch in line.chars() {
            if ch == '.' {
                matrix[i].push(Engine::Empty);
            } else if ch.is_digit(10) {
                matrix[i].push(Engine::Number(ch.to_digit(10).unwrap()));
            } else {
                matrix[i].push(Engine::Symbol(ch));
            }
        }
    }

    return matrix;
}

fn process_matrix(matrix: &Vec<Vec<Engine>>) -> u32 {
    let mut result = 0;

    for i in 0..matrix.len() {
        let mut is_num = false;
        let mut num = 0;
        let mut found_symbol = false;

        for j in 0..matrix[i].len() {
            match matrix[i][j] {
                Engine::Number(value) => {
                    if !found_symbol {
                        found_symbol = check_symbol(&matrix, i, j);
                    }

                    if is_num {
                        num = num * 10 + value;
                    } else {
                        num = value;
                    }

                    is_num = true;
                }
                _ => {
                    if is_num && found_symbol {
                        result += num;
                    }

                    is_num = false;
                    found_symbol = false;
                    num = 0;
                }
            }
        }

        if is_num && found_symbol {
            result += num;
        }
    }

    return result;
}

fn check_symbol(matrix: &Vec<Vec<Engine>>, x: usize, y: usize) -> bool {
    // There's probably a better way

    if x > 0 {
        if let Engine::Symbol(_) = matrix[x - 1][y] {
            return true;
        }

        if y > 0 {
            if let Engine::Symbol(_) = matrix[x - 1][y - 1] {
                return true;
            }
        }

        if y < matrix[x - 1].len() - 1 {
            if let Engine::Symbol(_) = matrix[x - 1][y + 1] {
                return true;
            }
        }
    }

    if y > 0 {
        if let Engine::Symbol(_) = matrix[x][y - 1] {
            return true;
        }
    }

    if y < matrix[x].len() - 1 {
        if let Engine::Symbol(_) = matrix[x][y + 1] {
            return true;
        }
    }

    if x < matrix.len() - 1 {
        if let Engine::Symbol(_) = matrix[x + 1][y] {
            return true;
        }

        if y > 0 {
            if let Engine::Symbol(_) = matrix[x + 1][y - 1] {
                return true;
            }
        }

        if y < matrix[x + 1].len() - 1 {
            if let Engine::Symbol(_) = matrix[x + 1][y + 1] {
                return true;
            }
        }
    }

    return false;
}
